/**
 * Propeller overlay demo.
 * Copyright (c) 2016 by Marco Maccaferri.
 *
 * MIT Licensed.
 */

#include <propeller.h>

#include "uart.h"
#include "eeprom.h"

void overlay_function1();
void overlay_function2();

extern uint32_t _ovly_table[2][3];

void eeprom_load_overlay(int n)
{
    uart_print("--- loading overlay ");
    uart_print_dec(n);
    uart_print(" from ");
    uart_print_number(_ovly_table[n][2], 16, 0);
    uart_print(" to ");
    uart_print_number(_ovly_table[n][0], 16, 0);
    uart_print(" size ");
    uart_print_dec(_ovly_table[n][1]);
    uart_print("\r\n");

    eeprom_read(HIGH_EEPROM_OFFSET(_ovly_table[n][2]), (uint8_t *)_ovly_table[n][0], _ovly_table[n][1]);
}

int main(int argc, char * argv[])
{
    int i;

    OUTA = (1 << I2C_SCL);
    DIRA = (1 << I2C_SCL);
    uart_start(31, 30, 0, 115200);

    DIRA &= ~(1 << I2C_SDA);                       // Set SDA as input
    for (i = 0; i < 9; i++) {
        OUTA &= ~(1 << I2C_SCL);                   // Put out up to 9 clock pulses
        OUTA |= (1 << I2C_SCL);
        if ((INA & (1 << I2C_SDA)) != 0)           // Repeat if SDA not driven high by the EEPROM
            break;
    }

    uart_print("\r\n*** OVERLAY DEMO ***\r\n\r\n");

    eeprom_load_overlay(0);
    overlay_function1();

    eeprom_load_overlay(1);
    overlay_function2();

    return 0;
}
