/**
 * Propeller overlay demo.
 * Copyright (c) 2016 by Marco Maccaferri.
 *
 * MIT Licensed.
 */

#include <propeller.h>

#include "uart.h"

#ifdef __GNUC__
#define INLINE__ static inline
#define Yield__() __asm__ volatile( "" ::: "memory" )
#else
#define INLINE__ static
static int32_t tmp__;
#define Yield__()
#endif

extern uint8_t _load_start_cog_uart_driver[];

struct _uart_data {

    // 9 contiguous longs
    volatile int32_t rx_head;
    volatile int32_t rx_tail;
    volatile int32_t tx_head;
    volatile int32_t tx_tail;
    volatile int32_t rx_pin;
    volatile int32_t tx_pin;
    volatile int32_t rxtx_mode;
    volatile int32_t bit_ticks;
    volatile int32_t buffer_ptr;

    // transmit and receive buffers
    volatile uint8_t rx_buffer[16];
    volatile uint8_t tx_buffer[16];
};

static struct _uart_data data;

int uart_start(int rxpin, int txpin, int mode, int baudrate)
{
    data.rx_head = 0;
    data.rx_tail = 0;
    data.tx_head = 0;
    data.tx_tail = 0;
    data.rx_pin = rxpin;
    data.tx_pin = txpin;
    data.rxtx_mode = mode;
    data.bit_ticks = CLKFREQ / baudrate;
    data.buffer_ptr = (int32_t) (&data.rx_buffer[0]);
    return cognew(_load_start_cog_uart_driver, (int32_t) &data) + 1;
}

void uart_write(int txbyte)
{
    while (!(data.tx_tail != ((data.tx_head + 1) & 0xf))) {
        Yield__();
    }
    data.tx_buffer[data.tx_head] = txbyte;
    data.tx_head = (data.tx_head + 1) & 0xf;
}

int uart_read_available()
{
    int rxbyte = -1;

    if (data.rx_tail != data.rx_head) {
        rxbyte = data.rx_buffer[data.rx_tail];
        data.rx_tail = (data.rx_tail + 1) & 0xf;
    }

    return rxbyte;
}

int uart_read()
{
    int rxbyte = 0;

    while ((rxbyte = uart_read_available()) < 0) {
        Yield__();
    }

    return rxbyte;
}

void uart_print(const char * str)
{
    while(*str) {
        uart_write(*str++);
    }
}

void uart_print_dec(unsigned long n)
{
    uart_print_number(n, 10, 0);
}

void uart_print_number(unsigned long n, int base, int digits)
{
    char buf[8 * sizeof(long) + 1]; // Assumes 8-bit chars plus zero byte.
    char *str = &buf[sizeof(buf) - 1];

    *str = '\0';

    // prevent crash if called with base == 1
    if (base < 2)
        base = 10;

    do {
        unsigned long m = n;
        n /= base;
        char c = m - base * n;
        *--str = c < 10 ? c + '0' : c + 'A' - 10;
        if (digits != -1)
            digits--;
    } while (n || (digits != -1 && digits > 0));

    return uart_print(str);
}
